@@ -95,16 +95,17 @@ class Params(object):
 
     def bind_param(self, value, name=None):
         if isinstance(value, Param):
-            name = value.placeholder
+            name = value.name
             value = value.value
-        elif value in self._bound_params.values():
+
+        if value in self._bound_params.values():
             for k, v in self._bound_params.items():
                 if v == value:
                     name = k
                     break
         elif value in self._bound_params.keys():
             for k, v in self._bound_params.items():
-                if v == k:
+                if k == value:
                     name = k
                     value = v
                     break
@@ -155,12 +156,12 @@ class _Link(type):
 class Pypher(with_metaclass(_Link)):
     PARAM_PREFIX = '$NEO'
 
-    def __init__(self, parent=None, *args, **kwargs):
+    def __init__(self, parent=None, params=None, *args, **kwargs):
         self._ = self
         self._parent = parent
         self.link = None
         self.next = None
-        self.params = Params(prefix=self.PARAM_PREFIX)
+        self.params = params or Params(prefix=self.PARAM_PREFIX)
 
     def reset(self):
         self.link = None
